﻿Imports System.Reflection

' Dieses Enumerable kann nur einmal durchlaufen werden, da der TextFieldParser 
'  nicht rückstellbar ist. Es ist geeignet für beliebig große Csv-Dateien, da 
'  immer nur eine Zeile eingelesen wird
Public Class CsvEnumerable(Of T As New) : Inherits Enumerable(Of T)

    Private _TP As FileIO.TextFieldParser

    Private Shared _Flds As FieldInfo() = _
        GetType(T).GetFields(BindingFlags.Instance Or BindingFlags.Public)
    Private Shared _Converters As System.ComponentModel.TypeConverter()

    Shared Sub New()
        ' Shared Sub New() - wird in der gesamten Laufzeit höchstens
        '  einmal durchlaufen
        ' Der VBCodeProvider liefert für jedes in T gefundene Feld den 
        '  geeigneten TypConverter
        Using Prov = New VBCodeProvider
            _Converters = _Flds.Select(Function(fld As FieldInfo) _
                Prov.GetConverter(fld.FieldType)).ToArray
        End Using
    End Sub

    Public Sub New(ByVal TP As FileIO.TextFieldParser)
        _TP = TP
    End Sub

    ''' <summary> 
    '''  Ermittelt das nächste Element der Enumeration 
    ''' </summary>
    ''' <returns> 
    '''  True: Element wurde ermittelt 
    '''  False: Enumeration beendet
    ''' </returns>
    Protected Overrides Function TryMovenext(ByRef Current As T) As Boolean
        TryMovenext = Not _TP.EndOfData
        If TryMovenext Then
            Current = New T
            Dim Row As String() = _TP.ReadFields

            ' Ausgelesene Werte nach Current übertragen
            For I = 0 To _Flds.Length - 1
                _Flds(I).SetValue(Current, _
                    _Converters(I).ConvertFromString(Row(I)))
            Next
        End If
    End Function
End Class
