Public Class Permutations

    ''' <summary> 
    '''  Erzeugt eine Enumeration aller Permutationen von InitArray 
    ''' </summary>
    Public Shared Function [Of](Of T)( _
        ByVal ParamArray InitArray As T()) As IEnumerable(Of T())

        Return From Indices In New IndexPermutations(InitArray.Length) _
            Select ApplyIndicees(InitArray, Indices)
    End Function

    ''' <summary>
    '''  Kopiert Arr in ein neues Array um, mit der durch Indices gegebenen 
    '''  Reihenfolge
    ''' </summary>
    Private Shared Function ApplyIndicees(Of T)( _
        ByVal Arr As T(), ByVal Indices As Integer()) As T()

        Dim RetVal(Arr.Length - 1) As T
        For I = 0 To Arr.Length - 1
            RetVal(I) = Arr(Indices(I))
        Next
        Return RetVal
    End Function

End Class 'Permutations

' Diese Enumerable ist quasi "virtuell", da immer dasselbe Array zurckgegeben 
'  wird
Public Class IndexPermutations : Inherits Enumerable(Of Integer())
    Private _Ubound As Integer

    Public Sub New(ByVal Length As Integer)
        _Ubound = Length - 1
    End Sub

    Protected Overrides Function TryMovenext( _
        ByRef Current() As Integer) As Boolean

        If Current Is Nothing Then
            ReDim Current(_Ubound)
            ' Initial-Indices: 0, 1, 2, 3,...
            For I = 0 To _Ubound
                Current(I) = I
            Next
            Return True
        End If

        ' Es wird ein linker und ein rechter Wert gesucht, die vertauscht 
        ' werden. Anschlieend die Reihenfolge rechts vom Links-Wert umkehren
        Dim Left As Integer, Right As Integer

        ' Links-Wert: muss kleiner sein als sein rechter Nachbar
        For Left = _Ubound - 1 To 0 Step -1
            If Current(Left) < Current(Left + 1) Then Exit For
        Next

        If Left < 0 Then
            ' Eine weitere Permutation kann nicht berechnet werden, Current 
            '  zurcksetzen - Damit sind die Aufrumarbeiten schon erledigt, und 
            '  Mybase.CleanUp braucht nicht berschrieben zu werden
            Current = Nothing
            Return False
        End If
        Dim LeftVal As Integer = Current(Left)

        ' Rechts-Wert: grer als der Links-Wert (ist nicht immer o.g. Nachbar!)
        For Right = _Ubound To 0 Step -1
            If Current(Right) > LeftVal Then Exit For
        Next

        ' Links-Wert und Rechts-Wert tauschen
        Current(Left) = Current(Right)
        Current(Right) = LeftVal

        ' Array rechts der Links-Position umkehren
        Array.Reverse(Current, Left + 1, _Ubound - Left)

        Return True
    End Function

End Class 'IndexPermutations
